
 /*--------------------------------------------------------------------------*/
 /*                 PRUDENS INC (c) 1996                                     */
 /*                 STREAMWOOD, ILLINOIS 60107  USA                          */
 /*                 ALL RIGHTS RESERVED.                                     */
 /*                 Class: SAMPLES                                           */
 /*--------------------------------------------------------------------------*/

 //
 //
 //   File :    DLG32.C
 //
 //   Purpose : This provides a sample for WUTL95.
 //             This is the 16 bit side.
 //
 //


/*---------------------------------------------------------------------------*/
/*                    C/Windows/PM Related Header Files                      */
/*---------------------------------------------------------------------------*/
#include <windows.h>


/*---------------------------------------------------------------------------*/
/*                      Product's Specific Header Files                      */
/*---------------------------------------------------------------------------*/
#include "smm.h"


/*---------------------------------------------------------------------------*/
/*                        Local Header Files                                 */
/*---------------------------------------------------------------------------*/
#include "common.h"


/*---------------------------------------------------------------------------*/
/*                        Predefined Constants                               */
/*---------------------------------------------------------------------------*/
#define MAX_HEAPSIZE    10240
#define MAX_STRLENGTH   1024


/*---------------------------------------------------------------------------*/
/*                        Local/Private Variables                            */
/*---------------------------------------------------------------------------*/
SMM_HANDLE  smmHeap;
LPSTR       lpPtr32;
LPSTR       lpPtr16;
HWND        hDlg16;


/*---------------------------------------------------------------------------*/
/*                        Local/Private APIs                                 */
/*---------------------------------------------------------------------------*/
BOOL WINAPI DlgProc32 ( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );


/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Class        : Samples                                                    */
/*                                                                           */
/* Method       : WinMain                                                    */
/*                                                                           */
/* Description  : stand Windows entry                                        */
/*                                                                           */
/* Misc.        :                                                            */
/*                                                                           */
/*---------------------------------------------------------------------------*/
int PASCAL WinMain ( HINSTANCE hCur, HINSTANCE hPrev, LPSTR lpCmd, INT nCmd )
{
    FARPROC     pfnDlgProc;

    // Prevent from starting the second instance
    if ( FindWindow ( NULL, DLG32_TITLE ) )
       return 0;

    // Setup share memory
    smmHeap = SMMCreate ( MAX_HEAPSIZE, SMM_ATTR_SHARABLE );
    if ( smmHeap == SMM_BAD_HEAP )
       return 0;

    if ( !( lpPtr32 = SMMAlloc ( smmHeap, MAX_STRLENGTH ) ) )
       return 0;

    // Get selector-based address
    lpPtr16 = SMML2S ( smmHeap, lpPtr32 );

    /*---------------------------------------------------------------------*/
    /* IMPORTANT NOTE:                                                     */
    /*          lpPtr32 is accessible from ANY 32 bit programs             */
    /*          lpPtr16 is accessible from ANY 16 bit programs             */
    /*---------------------------------------------------------------------*/

    pfnDlgProc = MakeProcInstance ( DlgProc32, hCur );
    DialogBox ( hCur, "SMMDLG", GetDesktopWindow (), pfnDlgProc );
    FreeProcInstance ( pfnDlgProc );

    SMMFree ( smmHeap, lpPtr32 );
    SMMDestroy ( smmHeap );
    SendMessage ( hDlg16, WM_COMMAND, IDCANCEL, 0L );
    return 0L;
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Class        : Samples                                                    */
/*                                                                           */
/* Method       : DlgProc32                                                  */
/*                                                                           */
/* Description  : stand DlgProc                                              */
/*                                                                           */
/* Misc.        :                                                            */
/*                                                                           */
/*---------------------------------------------------------------------------*/
BOOL WINAPI DlgProc32 ( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    static      INT     nTimeOut = 10;

    switch ( uMsg )
    {
       case WM_INITDIALOG:
            SetWindowText ( hDlg, DLG32_TITLE );

            // Start 16 program
            WinExec ( "Dlg16.Exe", SW_SHOWNORMAL );

            // Set timer, check every 0.5 second
            SetTimer ( hDlg, 101, 500, 0 );
            return TRUE;

       case WM_TIMER:
            nTimeOut--;
            if ( ! (hDlg16 = FindWindow ( NULL, DLG16_TITLE ) ) )
            {
               if ( nTimeOut < 0 )
               {
                  KillTimer ( hDlg, 101 );
                  MessageBox ( 0, "Can not start 16bit side", "ERROR", MB_OK | MB_ICONSTOP );
                  EndDialog ( hDlg, 0 );
               }
            }
            else
            {
               // Connect shared memory
               SendMessage ( hDlg16, WM_COMMAND, IDC_SELECTOR, HIWORD (lpPtr16) );
               SendMessage ( hDlg16, WM_COMMAND, IDC_OFFSET, LOWORD (lpPtr16) );
               KillTimer ( hDlg, 101 );
            }
            return TRUE;

       case WM_COMMAND:
            switch ( LOWORD (wParam) )
            {
               case IDCANCEL:
                    if ( hDlg16 )
                    {
                       SendMessage ( hDlg16, WM_COMMAND, IDC_SELECTOR, 0L );
                       SendMessage ( hDlg16, WM_COMMAND, IDC_OFFSET, 0L );
                    }
                    EndDialog ( hDlg, 0 );
                    break;

               case IDC_REFRESH:
                    SetDlgItemText ( hDlg, IDC_EDITBOX, lpPtr32 );
                    break;

               case IDC_EDITBOX:
                    switch ( HIWORD (wParam) )
                    {
                       case EN_CHANGE:
                            GetWindowText ( (HWND) lParam, lpPtr32, MAX_STRLENGTH );

                            if ( (HWND) lParam == GetFocus () )
                               SendMessage ( hDlg16, WM_COMMAND, IDC_REFRESH, 0L );
                            break;
                    }
                    break;
            }

            return TRUE;
    }

    return FALSE;
}


