unit mem32_0;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls;

const
  WM_SENDSELECTOROFFSET = WM_USER + 101;
  WM_RECEIVEMSG         = WM_USER + 102;

type
  TMainForm = class(TForm)
    SelectorOffsetLabel: TLabel;
    FromMem16Label: TLabel;
    FromMem16Edit: TEdit;
    ToMem16Label: TLabel;
    ToMem16Edit: TEdit;
    SendButton: TButton;
    CloseBitBtn: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SendButtonClick(Sender: TObject);
    procedure CloseBitBtnClick(Sender: TObject);
  private
    SMMHandle, LinearAddress, SelectorOffset: DWord;
  public
    procedure MessageWM_RECEIVEMSG (var Message: TMsg); message WM_RECEIVEMSG;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

const
  SMM_ATTR_EXTENSIBLE = $0003;
  SMM_ATTR_ZEROINIT   = $1000;
function SMMCreate
  (SharedMemSize: DWord; SharedMemAttr: DWord): DWord;
    stdcall; external 'SMMLIT95.DLL' index 111;
function SMMDestroy
  (SMMHandle: DWord): WordBool;
    stdcall; external 'SMMLIT95.DLL' index 112;
function SMMAlloc
  (SMMHandle: DWord; AllocSize: DWord): DWord;
    stdcall; external 'SMMLIT95.DLL' index 113;
function SMML2S
  (SMMHandle: DWord; LinearAddress: DWord): DWord;
    stdcall; external 'SMMLIT95.DLL' index 122;

procedure
  TMainForm.FormCreate
    (Sender: TObject);
begin
  // Create a shared area of 1000 bytes and initialize it with nulls;
  // there is another routine to allocate memory > 64k.
  SMMHandle := SMMCreate (1000, SMM_ATTR_EXTENSIBLE + SMM_ATTR_ZEROINIT);
  if SMMHandle <> -1 then
    // Allocate 100 bytes of the shared area as a test;
    // additional linear addresses can be created as needed.
    // The linear address is also available to any 32-bit application.
    LinearAddress := SMMAlloc (SMMHandle, 100);
  if LinearAddress <> 0 then begin
    // Translate the linear address to selector:offset pair
    SelectorOffset := SMML2S (SMMHandle, LinearAddress);
    // Display the selector:offset pair and linear address.
    SelectorOffsetLabel.Caption :=
      Format ('%.4x:%.4x (0x%.8x)',
              [HIWORD (SelectorOffset), LOWORD (SelectorOffset), LinearAddress]);
  end;
end;

procedure
  TMainForm.FormDestroy
    (Sender: TObject);
var
  Mem16Handle: HWnd;
begin
  // Tell Mem16 this shared memory area is being destroyed.
  Mem16Handle := FindWindow (nil, 'Mem16');
  if Mem16Handle <> 0 then
    SendMessage (Mem16Handle, WM_SENDSELECTOROFFSET, 0, 0);
  // Destroy the shared memory area.
  SMMDestroy (SMMHandle);
end;

procedure
  TMainForm.SendButtonClick
    (Sender: TObject);
var
  Mem16Handle: HWnd;
begin
// Make certain there is something to send.
  if Length (ToMem16Edit.Text) > 0 then begin
    // The linear address is a pointer to some memory; treat it like a PChar.
    StrPCopy (PChar (Pointer (LinearAddress)), ToMem16Edit.Text);
    // Find a window to send the selector:offset pair to.
    // Any 16-bit application can receive and use the selector:offset pair.
    Mem16Handle := FindWindow (nil, 'Mem16');
    if Mem16Handle <> 0 then
      SendMessage (Mem16Handle,
                   WM_SENDSELECTOROFFSET,
                   0,
                   MAKELPARAM (LOWORD (SelectorOffset), HIWORD (SelectorOffset)));
  end;
end;

procedure
  TMainForm.CloseBitBtnClick
    (Sender: TObject);
begin
  Close;
end; {TForm1.BitBtn1Click}

procedure
  TMainForm.MessageWM_RECEIVEMSG
    (var Message: TMsg);
begin
  // Copy to text from the shared memory area into the edit control.
  FromMem16Edit.Text := StrPas (PChar (LinearAddress));
end; {TForm1.MessageWM_RECEIVEMSG}

end. {mem32_0}
